/*******************************************************************************
*  Melody2 - 圧電スピーカからメロディを鳴らします                              *
*            EEPROMからメロディデータを読み込んでメロディを鳴らします          *
*            （EEPROMには予めデータが書き込んでいないと動作しません）          *
*            スイッチの替りにアナログ入力(5,4番端子)を使用しています。         *
*            ５番端子に５Ｖを接続したらスイッチ１がＯＮです。                  *
*            ４番端子に５Ｖを接続したらスイッチ２がＯＮです。                  *
*            ５、４番端子未接続でスイッチ１，２共にＯＦＦです。                *
*                                                                              *
* ============================================================================ *
*  VERSION DATE        BY                    CHANGE/COMMENT                    *
* ---------------------------------------------------------------------------- *
*  1.00    2011-03-30  きむ茶工房(きむしげ)  Create                            *
*******************************************************************************/
#include <EEPROM.h>

#define PINNO 12              // 圧電スピーカを接続したピン番号

int MelodyNote[17] = {0,1,2,4,8,16,32,64,128,1,2,4,8,16,32,64,128} ;
int sw1 ;
int sw2 ;

/*******************************************************************************
*  電源起動時とリセットの時だけのみ処理される関数(初期化と設定処理)            *
*******************************************************************************/
void setup() {
     pinMode(12,OUTPUT) ;     // 圧電スピーカを接続したピンをデジタル出力で指定
}
/*******************************************************************************
*  繰り返し実行される処理の関数(メインの処理)                                  *
*******************************************************************************/
void loop() {
     int adrs ;

     sw1 = 0 ;
     sw2 = 0 ;
     if (analogRead(5) > 1000) {        //スイッチ１の状態を調べる(スイッチ１優先)
          sw1 = 1 ;
          sw2 = 0 ;
     } else {
          if (analogRead(4) > 1000) {   //スイッチ２の状態を調べる
               sw1 = 0 ;
               sw2 = 1 ;
          }
     }
     if (sw1 == 1) {                    // メロディ１を鳴らす
          adrs = 0x00 ;
          MelodyOut(adrs) ;
     }
     if (sw2 == 1) {                    // メロディ２を鳴らす
          adrs = 0x20 ;
          MelodyOut(adrs) ;
     }
     delay(5000) ;                      // 5秒後に繰り返す
}
/*******************************************************************************
*  MelodyOut(adrs)                                                             *
*    EEPROMの指定したアドレスからメロディデータを読出します                    *
*                                                                              *
*    adrs:データの開始位置アドレスを指定                                       *
*******************************************************************************/
void MelodyOut(int adrs)
{
     unsigned int dt1 , dt2  ;
     long note ;
     int  temp ;
     int  scale ;
     int  i ;

     i = adrs ;
     temp = EEPROM.read(i) ;                 // テンポを取り出す
     i++ ;
     do {
          // EEPROMから取り出す
          dt1  = EEPROM.read(i) ;
          if (dt1 == 0xff) break ;           // メロディデータ終了
          dt2  = EEPROM.read(i+1) ;
          note = dt2 & 0x1f ;                // 音符を設定
          // 音符の長さを時間(us)に変換 note=全音符/note
          note = ((60000000L/temp)*4) / MelodyNote[note] ;
          if (note >= 9) {
               if (note <= 16) {
                    note = note + (note/2) ; // 付点音符処理
               }
          }
          dt2  = dt2 >> 5 ;
          scale = (dt1 << 3) | (dt2 & 0x7) ; // 音階(周波数値)を設定
          // 実際にメロディを出力する
          melody(scale,note) ;
          i = i + 2 ;
     } while(1) ;
}
/*******************************************************************************
*  melody(frequency,note)                                                      *
*    メロディを実際に鳴らす処理の関数                                          *
*    １サイクルのパルスを作成して、それを音の長さだけ繰り返し出力しています。  *
*                                                                              *
*    frequency:周波数(Hz)値で音の高さを指定する数値です                        *
*    note     :音の長さ(us)                                                    *
*******************************************************************************/
void melody(int frequency, long note)
{
     long tone ;
     long i , len ;

     // 指定された周波数から１サイクルの長さを計算する
     tone = (1000000L / frequency) / 2 ;
     // 音の長さ分のサイクル数を計算する
     len   = note / (1000000L / frequency) ;
     // 同じ長さのパルス(１サイクル)を音の長さ分繰返す
     for (i = 0; i < len ; i++) {
          digitalWrite(PINNO, HIGH) ;   // ブザーＯＮ
          delayMicroseconds(tone) ;
          digitalWrite(PINNO, LOW) ;    // ブザーＯＦＦ
          delayMicroseconds(tone) ;
     }
}
